/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.gui.options;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import ca.uvic.csr.shrimp.DisplayBean.DisplayBean;
import ca.uvic.csr.shrimp.DisplayBean.MagnifyZoomHandler;
import ca.uvic.csr.shrimp.DisplayBean.PNestedDisplayBean.PNormalZoomHandler;
import ca.uvic.csr.shrimp.ShrimpApplication.BeanNotFoundException;
import ca.uvic.csr.shrimp.ShrimpApplication.ShrimpToolNotFoundException;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool;
import edu.umd.cs.piccolo.util.PPaintContext;

/**
 * This Dialog lets the user choose rendering options, 
 * such as the speed the screen will move at when zooming.
 *
 * @author YiLing Lu, Rob Lintern
 * date: May 07, 2001
 */
public class RenderingOptionsPanel extends JPanel implements ShrimpOptions {

	protected String toolName;
	protected ShrimpProject project;
	
	protected JSlider magnifySlider;
	protected JSlider zoomSlider;
	private JRadioButton rdoStaticRendQualLow;
	private JRadioButton rdoStaticRendQualHigh;
	private ButtonGroup grpStaticRendQual;

	private JRadioButton rdoDynamicRendQualLow;
	private JRadioButton rdoDynamicRendQualHigh;
	private ButtonGroup grpDynamicRendQual;

	private int staticRendQual;
	private int dynamicRendQual;

	public int magnifySelectedSpeed;
	public int zoomSelectedSpeed;

	/**
	 * Constructor for RenderingOptionsPanel
	 * @param toolName The name of the tool to apply the rendering options to.
	 */
	public RenderingOptionsPanel(String toolName, ShrimpProject project) {
		this.toolName = toolName;
		this.project = project;
		initGUI();
	}

	protected void initGUI() {
		// rendering quality

		JPanel pnlStaticRendQual = new JPanel();
		pnlStaticRendQual.setBorder(BorderFactory.createEtchedBorder());
		{
			staticRendQual = PPaintContext.HIGH_QUALITY_RENDERING;
			try {
				DisplayBean displayBean = (DisplayBean) project.getTool(toolName).getBean(ShrimpTool.DISPLAY_BEAN);
				staticRendQual = displayBean.getStaticRenderingQuality();
			} catch (BeanNotFoundException e1) {
				// nothing
			} catch (ShrimpToolNotFoundException e1) {
				//nothing
			}

			JLabel lblrendQualTitle = new JLabel("Static Rendering Quality: ");
			rdoStaticRendQualLow = new JRadioButton("Low");
			rdoStaticRendQualHigh = new JRadioButton("High");
			grpStaticRendQual = new ButtonGroup();
			grpStaticRendQual.add(rdoStaticRendQualLow);
			grpStaticRendQual.add(rdoStaticRendQualHigh);
			switch (staticRendQual) {
				case PPaintContext.LOW_QUALITY_RENDERING :
					rdoStaticRendQualLow.setSelected(true);
					break;
				case PPaintContext.HIGH_QUALITY_RENDERING :
					rdoStaticRendQualHigh.setSelected(true);
					break;

				default :
					rdoStaticRendQualHigh.setSelected(true);
					break;
			}

			ActionListener rdoStaticActionListener = new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (rdoStaticRendQualLow.isSelected()) {
						staticRendQual = PPaintContext.LOW_QUALITY_RENDERING;
					} else {
						staticRendQual = PPaintContext.HIGH_QUALITY_RENDERING;
					}
				}
			};
			rdoStaticRendQualLow.addActionListener(rdoStaticActionListener);
			rdoStaticRendQualHigh.addActionListener(rdoStaticActionListener);

			pnlStaticRendQual.add(lblrendQualTitle);
			pnlStaticRendQual.add(rdoStaticRendQualLow);
			pnlStaticRendQual.add(rdoStaticRendQualHigh);
		}

		// interacting rendering quality		
		JPanel pnlDynamicRendQual = new JPanel();
		pnlDynamicRendQual.setBorder(BorderFactory.createEtchedBorder());
		dynamicRendQual = PPaintContext.HIGH_QUALITY_RENDERING;
		try {
			DisplayBean displayBean = (DisplayBean) project.getTool(toolName).getBean(ShrimpTool.DISPLAY_BEAN);
			dynamicRendQual = displayBean.getDynamicRenderingQuality();
		} catch (BeanNotFoundException e1) {
			// nothing
		} catch (ShrimpToolNotFoundException e1) {
			//nothing
		}

		JLabel lblrendQualTitle = new JLabel("Dynamic Rendering Quality: ");
		rdoDynamicRendQualLow = new JRadioButton("Low");
		rdoDynamicRendQualHigh = new JRadioButton("High");
		grpDynamicRendQual = new ButtonGroup();
		grpDynamicRendQual.add(rdoDynamicRendQualLow);
		grpDynamicRendQual.add(rdoDynamicRendQualHigh);
		switch (dynamicRendQual) {
			case PPaintContext.LOW_QUALITY_RENDERING :
				rdoDynamicRendQualLow.setSelected(true);
				break;
			case PPaintContext.HIGH_QUALITY_RENDERING :
				rdoDynamicRendQualHigh.setSelected(true);
				break;
			default :
				rdoDynamicRendQualHigh.setSelected(true);
				break;
		}

		ActionListener rdoDynamicActionListener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (rdoDynamicRendQualLow.isSelected()) {
					dynamicRendQual = PPaintContext.LOW_QUALITY_RENDERING;
				} else {
					dynamicRendQual = PPaintContext.HIGH_QUALITY_RENDERING;
				}
			}
		};
		rdoDynamicRendQualLow.addActionListener(rdoDynamicActionListener);
		rdoDynamicRendQualHigh.addActionListener(rdoDynamicActionListener);

		pnlDynamicRendQual.add(lblrendQualTitle);
		pnlDynamicRendQual.add(rdoDynamicRendQualLow);
		pnlDynamicRendQual.add(rdoDynamicRendQualHigh);

		// setup the magnify slider
		JPanel magnifySliderPanel = new JPanel();
		magnifySliderPanel.setBorder(BorderFactory.createEtchedBorder());

		magnifySelectedSpeed = MagnifyZoomHandler.getAnimationTime();
		magnifySlider = new JSlider(JSlider.HORIZONTAL, 0, 5000, magnifySelectedSpeed);
		magnifySlider.setMajorTickSpacing(1000);
		magnifySlider.setPaintTicks(true);
		magnifySlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				if (!source.getValueIsAdjusting()) {
					magnifySelectedSpeed = source.getValue();
				}
			}
		});
		JLabel magfifyTitle = new JLabel("Magnify Speed (in seconds): ");
		magnifySliderPanel.add(magfifyTitle);
		magnifySliderPanel.add(new JLabel("0.0"));
		magnifySliderPanel.add(magnifySlider);
		magnifySliderPanel.add(new JLabel("5.0"));

		// set up the zoom slider
		JPanel zoomSliderPanel = new JPanel();
		zoomSliderPanel.setBorder(BorderFactory.createEtchedBorder());

		zoomSelectedSpeed = PNormalZoomHandler.getZoomSpeed();
		zoomSlider = new JSlider(JSlider.HORIZONTAL, PNormalZoomHandler.MIN_ZOOM_PAUSE, PNormalZoomHandler.MAX_ZOOM_PAUSE, zoomSelectedSpeed);
		zoomSlider.setMajorTickSpacing(25);
		zoomSlider.setPaintTicks(true);
		zoomSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				if (!source.getValueIsAdjusting()) {
					zoomSelectedSpeed = source.getValue();
				}
			}
		});
		JLabel zoomTitle = new JLabel("Zoom Speed: ");
		zoomSliderPanel.add(zoomTitle);
		zoomSliderPanel.add(new JLabel("Fast"));
		zoomSliderPanel.add(zoomSlider);
		zoomSliderPanel.add(new JLabel("Slow"));

		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		add(pnlStaticRendQual);
		add(pnlDynamicRendQual);
		add(magnifySliderPanel);
		add(zoomSliderPanel);
	}

	public void ok() {
		MagnifyZoomHandler.setAnimationTime(magnifySelectedSpeed);
		PNormalZoomHandler.setZoomSpeed(zoomSelectedSpeed);
		try {
			DisplayBean displayBean = (DisplayBean) project.getTool(toolName).getBean(ShrimpTool.DISPLAY_BEAN);
			displayBean.setStaticRenderingQuality(staticRendQual);
			displayBean.setDynamicRenderingQuality(dynamicRendQual);
		} catch (BeanNotFoundException e1) {
			// nothing
		} catch (ShrimpToolNotFoundException e1) {
			//nothing
		}
	}

	public void cancel() {
	}

}
